/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework;



import java.util.HashSet;
import java.util.Set;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.ArrayBehaviour;
import com.tocea.easycoverage.framework.behaviour.GetterSetterBehaviour;
import com.tocea.easycoverage.framework.behaviour.InstantiationBehaviour;
import com.tocea.easycoverage.framework.behaviour.NPEConstructorBehaviour;
import com.tocea.easycoverage.framework.behaviour.NPEMethodBehaviour;
import com.tocea.easycoverage.framework.behaviour.SimpleInvocationBehaviour;
import com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour;



/**
 * This type describes the automatic test
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 12 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */
public final class TestAutomatic
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER            = LoggerFactory.getLogger(TestAutomatic.class);
    
    
    private static final Class[]          AVAILABLE_PLUGINS = {
            InstantiationBehaviour.class,
            ArrayBehaviour.class,
            NPEConstructorBehaviour.class,
            NPEMethodBehaviour.class,
            GetterSetterBehaviour.class,
            SimpleInvocationBehaviour.class                };
    
    
    
    /**
     * This method returns the available plugins.
     * 
     * @return the available plugins.
     */
    public static Class<ITestBehaviour>[] getAvailablePlugins() {
    
    
        return TestAutomatic.AVAILABLE_PLUGINS;
    }
    
    
    /**
     * This method returns a new class scanner.
     * 
     * @return a new class scanner.
     */
    public static ClassScanner newClassScanner() {
    
    
        return new ClassScanner();
        
    }
    
    
    /**
     * This method performs the creation of a test harness.
     * 
     * @param _plugins
     *            the plugins
     * @return the new test harness
     */
    public static TestHarness newTestHarness(final Class<? extends ITestBehaviour>... _plugins) {
    
    
        if (_plugins == null) {
            return newTestHarness(AVAILABLE_PLUGINS);
        }
        final Set<Class<? extends ITestBehaviour>> plugins = new HashSet<Class<? extends ITestBehaviour>>();
        for (final Class<? extends ITestBehaviour> str : _plugins) {
            if (str == null) {
                continue; // Skip null classes
            }
            plugins.add(str);
        }
        return newTestHarness(plugins);
    }
    
    
    /**
     * This method produces a test harness with the given plugins.
     * 
     * @param _plugins
     *            the plugin names
     * @return the test harness
     */
    private static TestHarness newTestHarness(final Set<Class<? extends ITestBehaviour>> _plugins) {
    
    
        if (_plugins == null) {
            return null;
        }
        
        final TestHarness testHarness = new TestHarness();
        for (final Class<? extends ITestBehaviour> behaviour : _plugins) {
            try {
                testHarness.addBehaviour(behaviour.newInstance());
            } catch (final InstantiationException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final IllegalAccessException e) {
                LOGGER.error(e.getMessage(), e);
                
            }
        }
        return testHarness;
    }
    
    
    private TestAutomatic() {
    
    
        super();
        
    }
}
