/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.behaviour;



import java.lang.reflect.Modifier;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;
import com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour;
import com.tocea.easycoverage.framework.unit.NPEConstructorTest;




/**
 * This type describes the instantiation behaviour.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 12 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class NPEConstructorBehaviour implements ITestBehaviour
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER                  = LoggerFactory.getLogger(NPEConstructorBehaviour.class);
    
    
    /**
     * This field defines the modifiers to exclude methods in the analysis.
     */
    public static int                     excludeMethodsModifiers = Modifier.PRIVATE | Modifier.STATIC;
    
    public static Class[]                 ignoreImplementations   = {
                                                                      Throwable.class };
    
    
    
    /**
     * Builds a instantiation behaviour.
     */
    public NPEConstructorBehaviour() {
    
    
        super();
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour#newInstanceTest(java.lang.Object)
     */
    public IClassTest newInstanceTest(final Object _testedClass) {
    
    
        return null;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour#newTest(java.lang.Class)
     */
    public IClassTest newTest(final Class<?> _testedClass) {
    
    
        if (_testedClass == null) {
            return null;
        }
        for (final Class<?> ignoreImpl : ignoreImplementations) {
            if (ignoreImpl.isAssignableFrom(_testedClass)) {
                return null;
            }
        }
        return new NPEConstructorTest(_testedClass, excludeMethodsModifiers);
        
    }
    
}
